﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AudioTools
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        //---------------------------------------------------------------------
        //  フィールド
        //---------------------------------------------------------------------
        private System.Media.SoundPlayer _player = new System.Media.SoundPlayer();
        private List<short> _waveData = null;

        //---------------------------------------------------------------------
        //  コンストラクタ
        //---------------------------------------------------------------------
        public MainWindow()
        {
            InitializeComponent();
        }

        //---------------------------------------------------------------------
        //  波形描画
        //---------------------------------------------------------------------
        private void DrawWave()
        {
            //  波形グリッドの子パーツをクリア
            waveGrid.Children.Clear();

            //  波形データがなければ終了
            if (_waveData == null)
            {
                return;
            }

             //  センターライン(Line)をグリッドに追加
            double waveHeight = waveGrid.ActualHeight / 2;
            System.Windows.Shapes.Line centerLine = new System.Windows.Shapes.Line();
            centerLine.Stroke = Brushes.Red;
            centerLine.StrokeThickness = 1;
            waveGrid.Children.Add(centerLine);
            centerLine.X1 = 0;
            centerLine.X2 = waveGrid.ActualWidth;
            centerLine.Y1 = waveHeight;
            centerLine.Y2 = waveHeight;

            //  波形ライン(Polyline)をグリッドに追加
            System.Windows.Shapes.Polyline waveLine = new Polyline();
            waveLine.Stroke = Brushes.Black;
            waveLine.StrokeThickness = 1;
            waveGrid.Children.Add(waveLine);

            //  波形ラインの座標を追加
            for (int i = 0; i < waveGrid.ActualWidth; i++)
            {
                if (i >= _waveData.Count)
                {
                    break;
                }

                //  部分表示
                //waveLine.Points.Add(new Point(i, waveHeight - (waveHeight * (_waveData[i] / (double)Int16.MaxValue))));
            
                //  全体表示
                var rangeLength = (int)(_waveData.Count / waveGrid.ActualWidth);
                var count = rangeLength;
                if (i * rangeLength + count >= _waveData.Count)
                {
                    count = _waveData.Count - (i * rangeLength) - 1;
                }
                if (count > 0)
                {
                    var max = _waveData.GetRange(i * rangeLength, count).Max();
                    var min = _waveData.GetRange(i * rangeLength, count).Min();
                    waveLine.Points.Add(new Point(i, waveHeight - (waveHeight * (max / (double)Int16.MaxValue))));
                    waveLine.Points.Add(new Point(i, waveHeight - (waveHeight * (min / (double)Int16.MaxValue))));
                }
            }
        }

        //---------------------------------------------------------------------
        //  イベント：再生ボタン クリック
        //---------------------------------------------------------------------
        private void playButton_Click(object sender, RoutedEventArgs e)
        {
            //  再生
            _player.Play();
        }

        //---------------------------------------------------------------------
        //  イベント：停止ボタン クリック
        //---------------------------------------------------------------------
        private void stopButton_Click(object sender, RoutedEventArgs e)
        {
            //  停止
            _player.Stop();
        }

        //---------------------------------------------------------------------
        //  イベント：開くボタン クリック
        //---------------------------------------------------------------------
        private void openButton_Click(object sender, RoutedEventArgs e)
        {
            //  ダイアログの作成
            Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog();
            dialog.Filter = "*.wav|*.wav";

            //  ダイアログを表示してファイルを選択
            if (dialog.ShowDialog() == true)
            {
                //  フォーマット情報の取得
                var info = WaveFormat.GetFormatInfo(dialog.FileName);
                if (info.IsValid)
                {
                    //  フォーマット情報を表示
                    formatTextBlock.Text =
                        dialog.FileName + Environment.NewLine +
                        info.Bit.ToString() + "bit  " +
                        info.SampleRate.ToString() + "Hz  " +
                        info.ChannelCount.ToString() + "Channel(s)" + Environment.NewLine +
                        info.Duration.ToString("0.00 sec");

                    //  波形データの取得
                    _waveData = WaveFormat.GetWaveData(dialog.FileName);

                    //  波形の描画
                    DrawWave();

                    //  サウンドファイルの指定
                    _player.SoundLocation = dialog.FileName;
                }
                else
                {
                    //  警告を表示
                    formatTextBlock.Text = "INVALID FORMAT !!";
                }
            }
        }
    }
}
